@@ -1,9 +1,9 @@
 /* Copyright 2003-2004 Roger Dingledine
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c,v 1.79 2005/12/14 20:40:39 nickm Exp $ */
+/* $Id: compat.c,v 1.82 2006/02/09 05:46:48 arma Exp $ */
 const char compat_c_id[] =
-  "$Id: compat.c,v 1.79 2005/12/14 20:40:39 nickm Exp $";
+  "$Id: compat.c,v 1.82 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file compat.c
@@ -441,17 +441,23 @@ tor_socketpair(int family, int type, int protocol, int fd[2])
 
 #define ULIMIT_BUFFER 32 /* keep 32 extra fd's beyond _ConnLimit */
 
-/** Get the maximum allowed number of file descriptors. (Some systems
- * have a low soft limit.) Make sure we set it to at least
- * <b>limit</b>. Return a new limit if we can, or -1 if we fail. */
+/** Learn the maximum allowed number of file descriptors. (Some systems
+ * have a low soft limit.
+ *
+ * We compute this by finding the largest number between <b>limit</b>
+ * and <b>cap</b> that we can use. If we can't find a number greater
+ * than or equal to <b>limit</b>, then we fail: return -1.
+ *
+ * Otherwise, return the number minus some buffer to allow for other
+ * file descriptors we'll want available for ordinary use. */
 int
 set_max_file_descriptors(unsigned long limit, unsigned long cap)
 {
 #ifndef HAVE_GETRLIMIT
   log_fn(LOG_INFO, LD_NET,
          "This platform is missing getrlimit(). Proceeding.");
-  if (limit > cap) {
-    log(LOG_INFO, LD_CONFIG, "ConnLimit must be at most %d. Capping it.", cap);
+  if (limit < cap) {
+    log(LOG_INFO, LD_CONFIG, "ConnLimit must be at most %d. Using that.", cap);
     limit = cap;
   }
 #else
