@@ -1,8 +1,9 @@
-/* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: log.c,v 1.111 2006/01/11 19:40:14 nickm Exp $ */
-const char log_c_id[] = "$Id: log.c,v 1.111 2006/01/11 19:40:14 nickm Exp $";
+/* $Id: log.c,v 1.113 2006/02/09 05:46:48 arma Exp $ */
+const char log_c_id[] = "$Id: log.c,v 1.113 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file log.c
@@ -487,7 +488,7 @@ add_syslog_log(int loglevelMin, int loglevelMax)
   logfile_t *lf;
   if (syslog_count++ == 0)
     /* This is the first syslog. */
-    openlog("Tor", LOG_PID | LOG_NDELAY, LOG_DAEMON);
+    openlog("Tor", LOG_PID | LOG_NDELAY, LOGFACILITY);
 
   lf = tor_malloc_zero(sizeof(logfile_t));
   lf->min_loglevel = loglevelMin;
