@@ -1,9 +1,9 @@
 /* Copyright 2003 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tortls.c,v 1.114 2006/01/10 21:12:06 nickm Exp $ */
+/* $Id: tortls.c,v 1.116 2006/02/09 05:46:48 arma Exp $ */
 const char tortls_c_id[] =
-  "$Id: tortls.c,v 1.114 2006/01/10 21:12:06 nickm Exp $";
+  "$Id: tortls.c,v 1.116 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file tortls.c
@@ -363,7 +363,7 @@ tor_tls_context_new(crypto_pk_env_t *identity,
       goto error;
     SSL_CTX_set_options(*ctx, SSL_OP_NO_SSLv2);
 #endif
-#ifndef ENABLE_0119_PARANOIA
+#ifndef ENABLE_0119_PARANOIA_A
     SSL_CTX_set_options(*ctx, SSL_OP_SINGLE_DH_USE);
 #endif
     if (!SSL_CTX_set_cipher_list(*ctx, CIPHER_LIST))
