@@ -1,10 +1,10 @@
-/* Copyright 2001 Matej Pfajfar.
- * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.209 2006/01/11 04:14:10 arma Exp $ */
+/* $Id: circuitbuild.c,v 1.214 2006/02/09 05:46:48 arma Exp $ */
 const char circuitbuild_c_id[] =
-  "$Id: circuitbuild.c,v 1.209 2006/01/11 04:14:10 arma Exp $";
+  "$Id: circuitbuild.c,v 1.214 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file circuitbuild.c
@@ -493,7 +493,7 @@ circuit_deliver_create_cell(circuit_t *circ, uint8_t cell_type, char *payload)
 /** We've decided to start our reachability testing. If all
  * is set, log this to the user. Return 1 if we did, or 0 if
  * we chose not to log anything. */
-static int
+int
 inform_testing_reachability(void)
 {
   char dirbuf[128];
@@ -610,6 +610,7 @@ circuit_send_next_onion_skin(circuit_t *circ)
             "Tor has successfully opened a circuit. Looks like it's working.");
         if (server_mode(options) && !check_whether_orport_reachable()) {
           inform_testing_reachability();
+          consider_testing_reachability();
         }
       }
       circuit_rep_hist_note_result(circ);
@@ -1202,7 +1203,8 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
   }
   info(LD_CIRC,
        "Found %d servers that might support %d/%d pending connections.",
-       n_best_support, best_support, n_pending_connections);
+       n_best_support, best_support >= 0 ? best_support : 0,
+       n_pending_connections);
 
   preferredexits = smartlist_create();
   add_nickname_list_to_smartlist(preferredexits,options->ExitNodes,1,1,1);
@@ -1696,7 +1698,7 @@ build_state_get_exit_nickname(cpath_build_state_t *state)
  * - Listed as either up or never yet contacted;
  * - Present in the routerlist;
  * - Listed as 'stable' or 'fast' by the current dirserver concensus,
- *   if demanded by <b>need_uptime</b> and <b>need_capacity</b>; and
+ *   if demanded by <b>need_uptime</b> or <b>need_capacity</b>; and
  * - Allowed by our current ReachableAddresses config option.
  */
 static INLINE routerinfo_t *
