@@ -1,10 +1,10 @@
-/* Copyright 2001 Matej Pfajfar.
- * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c,v 1.433 2006/01/10 22:42:44 arma Exp $ */
+/* $Id: connection.c,v 1.439 2006/02/09 05:46:48 arma Exp $ */
 const char connection_c_id[] =
-  "$Id: connection.c,v 1.433 2006/01/10 22:42:44 arma Exp $";
+  "$Id: connection.c,v 1.439 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file connection.c
@@ -337,6 +337,7 @@ connection_about_to_close_connection(connection_t *conn)
         /* It's a directory connection and connecting or fetching
          * failed: forget about this router, and maybe try again. */
         connection_dir_request_failed(conn);
+        // XXX if it's rend desc we may want to retry -RD
       }
       if (conn->purpose == DIR_PURPOSE_FETCH_RENDDESC)
         rend_client_desc_here(conn->rend_query); /* give it a try */
@@ -546,8 +547,12 @@ connection_create_listener(const char *listenaddress, uint16_t listenport,
 #endif
 
   if (bind(s,(struct sockaddr *)&listenaddr,sizeof(listenaddr)) < 0) {
-    warn(LD_NET, "Could not bind to %s:%u: %s", address, usePort,
-         tor_socket_strerror(tor_socket_errno(s)));
+    const char *helpfulhint = "";
+    int e = tor_socket_errno(s);
+    if (ERRNO_IS_EADDRINUSE(e))
+      helpfulhint = ". Is Tor already running?";
+    warn(LD_NET, "Could not bind to %s:%u: %s%s", address, usePort,
+         tor_socket_strerror(e), helpfulhint);
     goto err;
   }
 
@@ -983,13 +988,18 @@ static int
 connection_bucket_read_limit(connection_t *conn)
 {
   int at_most;
+  int base = connection_speaks_cells(conn) ?
+               CELL_NETWORK_SIZE : RELAY_PAYLOAD_SIZE;
 
-  /* do a rudimentary round-robin so one circuit can't hog a connection */
-  if (connection_speaks_cells(conn)) {
-    at_most = 32*(CELL_NETWORK_SIZE);
-  } else {
-    at_most = 32*(RELAY_PAYLOAD_SIZE);
-  }
+  /* Do a rudimentary round-robin so one circuit can't hog a connection.
+   * Pick at most 32 cells, at least 4 cells if possible, and if we're in
+   * the middle pick 1/8 of the available bandwidth. */
+  at_most = global_read_bucket / 8;
+  at_most -= (at_most % base); /* round down */
+  if (at_most > 32*base) /* 16 KB */
+    at_most = 32*base;
+  else if (at_most < 4*base) /* 2 KB */
+    at_most = 4*base;
 
   if (at_most > global_read_bucket)
     at_most = global_read_bucket;
@@ -2079,6 +2089,7 @@ assert_connection_ok(connection_t *conn, time_t now)
     case CONN_TYPE_OR:
       tor_assert(conn->state >= _OR_CONN_STATE_MIN);
       tor_assert(conn->state <= _OR_CONN_STATE_MAX);
+      tor_assert(conn->n_circuits >= 0);
       break;
     case CONN_TYPE_EXIT:
       tor_assert(conn->state >= _EXIT_CONN_STATE_MIN);
