@@ -1,10 +1,10 @@
-/* Copyright 2001 Matej Pfajfar.
- * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_edge.c,v 1.377 2006/01/05 21:23:03 nickm Exp $ */
+/* $Id: connection_edge.c,v 1.380 2006/02/09 05:46:48 arma Exp $ */
 const char connection_edge_c_id[] =
-  "$Id: connection_edge.c,v 1.377 2006/01/05 21:23:03 nickm Exp $";
+  "$Id: connection_edge.c,v 1.380 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file connection_edge.c
@@ -924,15 +924,205 @@ addressmap_get_mappings(smartlist_t *sl, time_t min_expires,
    }
 }
 
+/* Connection <b>conn</b> just finished its socks handshake, or the
+ * controller asked us to take care of it.
+ *
+ * First, parse whether it's a .exit address, remap it, and so on. Then
+ * it's for a general circuit, try to attach it to a circuit (or launch
+ * one as needed), else if it's for a rendezvous circuit, fetch a
+ * rendezvous descriptor first (or attach/launch a circuit if the
+ * rendezvous descriptor is already here and fresh enough).
+ */
+int
+connection_ap_handshake_rewrite_and_attach(connection_t *conn)
+{
+  socks_request_t *socks = conn->socks_request;
+  hostname_type_t addresstype;
+
+  tor_strlower(socks->address); /* normalize it */
+  debug(LD_APP,"Client asked for %s:%d", safe_str(socks->address),
+        socks->port);
+
+  /* For address map controls, remap the address */
+  addressmap_rewrite(socks->address, sizeof(socks->address));
+
+  if (address_is_in_virtual_range(socks->address)) {
+    /* This address was probably handed out by client_dns_get_unmapped_address,
+     * but the mapping was discarded for some reason.  We *don't* want to send
+     * the address through Tor; that's likely to fail, and may leak
+     * information.
+     */
+    warn(LD_APP,"Missing mapping for virtual address '%s'. Refusing.",
+         socks->address); /* don't safe_str() this yet. */
+    connection_mark_unattached_ap(conn, END_STREAM_REASON_INTERNAL);
+    return -1;
+  }
+
+  /* Parse the address provided by SOCKS.  Modify it in-place if it
+   * specifies a hidden-service (.onion) or particular exit node (.exit).
+   */
+  addresstype = parse_extended_hostname(socks->address);
+
+  if (addresstype == BAD_HOSTNAME) {
+    warn(LD_APP, "Invalid hostname %s; rejecting", socks->address);
+    connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
+    return -1;
+  }
+
+  if (addresstype == EXIT_HOSTNAME) {
+    /* foo.exit -- modify conn->chosen_exit_node to specify the exit
+     * node, and conn->address to hold only the address portion.*/
+    char *s = strrchr(socks->address,'.');
+    if (s) {
+      if (s[1] != '\0') {
+        conn->chosen_exit_name = tor_strdup(s+1);
+        *s = 0;
+      } else {
+        warn(LD_APP,"Malformed exit address '%s.exit'. Refusing.",
+             safe_str(socks->address));
+        connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
+        return -1;
+      }
+    } else {
+      struct in_addr in;
+      routerinfo_t *r;
+      conn->chosen_exit_name = tor_strdup(socks->address);
+      r = router_get_by_nickname(conn->chosen_exit_name, 1);
+      *socks->address = 0;
+      if (r) {
+        /* XXXX Should this use server->address instead? */
+        in.s_addr = htonl(r->addr);
+        strlcpy(socks->address, inet_ntoa(in), sizeof(socks->address));
+      } else {
+        warn(LD_APP,
+             "Unrecognized server in exit address '%s.exit'. Refusing.",
+             safe_str(socks->address));
+        connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
+        return -1;
+      }
+    }
+  }
+
+  if (addresstype != ONION_HOSTNAME) {
+    /* not a hidden-service request (i.e. normal or .exit) */
+
+    if (address_is_invalid_destination(socks->address)) {
+      warn(LD_APP,"Destination '%s' seems to be an invalid hostname. Failing.",
+           safe_str(socks->address));
+      connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
+      return -1;
+    }
+
+    if (socks->command == SOCKS_COMMAND_RESOLVE) {
+      uint32_t answer;
+      struct in_addr in;
+      /* Reply to resolves immediately if we can. */
+      if (strlen(socks->address) > RELAY_PAYLOAD_SIZE) {
+        warn(LD_APP,"Address to be resolved is too large. Failing.");
+        connection_ap_handshake_socks_resolved(conn,RESOLVED_TYPE_ERROR,
+                                               0,NULL,-1);
+        connection_mark_unattached_ap(conn,
+                                      END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
+        return -1;
+      }
+      if (tor_inet_aton(socks->address, &in)) { /* see if it's an IP already */
+        answer = in.s_addr; /* leave it in network order */
+        connection_ap_handshake_socks_resolved(conn,RESOLVED_TYPE_IPV4,4,
+                                               (char*)&answer,-1);
+        connection_mark_unattached_ap(conn,
+                                      END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
+        return 0;
+      }
+      rep_hist_note_used_resolve(time(NULL)); /* help predict this next time */
+    } else { /* socks->command == SOCKS_COMMAND_CONNECT */
+      if (socks->port == 0) {
+        notice(LD_APP,"Application asked to connect to port 0. Refusing.");
+        connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
+        return -1;
+      }
+
+      if (!conn->chosen_exit_name) {
+        /* see if we can find a suitable enclave exit */
+        routerinfo_t *r =
+          router_find_exact_exit_enclave(socks->address, socks->port);
+        if (r) {
+          info(LD_APP,"Redirecting address %s to exit at enclave router %s",
+               safe_str(socks->address), r->nickname);
+          /* use the hex digest, not nickname, in case there are two
+             routers with this nickname */
+          conn->chosen_exit_name =
+            tor_strdup(hex_str(r->cache_info.identity_digest, DIGEST_LEN));
+        }
+      }
+
+      /* help predict this next time */
+      rep_hist_note_used_port(socks->port, time(NULL));
+    }
+    conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
+    if (connection_ap_handshake_attach_circuit(conn) < 0) {
+      connection_mark_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
+      return -1;
+    }
+    return 0;
+  } else {
+    /* it's a hidden-service request */
+    rend_cache_entry_t *entry;
+    int r;
+
+    if (socks->command == SOCKS_COMMAND_RESOLVE) {
+      /* if it's a resolve request, fail it right now, rather than
+       * building all the circuits and then realizing it won't work. */
+      warn(LD_APP,"Resolve requests to hidden services not allowed. Failing.");
+      connection_ap_handshake_socks_resolved(conn,RESOLVED_TYPE_ERROR,
+                                             0,NULL,-1);
+      connection_mark_unattached_ap(conn,
+                                    END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
+      return -1;
+    }
+
+    strlcpy(conn->rend_query, socks->address, sizeof(conn->rend_query));
+    info(LD_REND,"Got a hidden service request for ID '%s'",
+         safe_str(conn->rend_query));
+    /* see if we already have it cached */
+    r = rend_cache_lookup_entry(conn->rend_query, -1, &entry);
+    if (r<0) {
+      warn(LD_BUG,"Bug: Invalid service name '%s'",
+           safe_str(conn->rend_query));
+      connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
+      return -1;
+    }
+    if (r==0) {
+      conn->state = AP_CONN_STATE_RENDDESC_WAIT;
+      info(LD_REND, "Unknown descriptor %s. Fetching.",
+           safe_str(conn->rend_query));
+      rend_client_refetch_renddesc(conn->rend_query);
+    } else { /* r > 0 */
+#define NUM_SECONDS_BEFORE_REFETCH (60*15)
+      if (time(NULL) - entry->received < NUM_SECONDS_BEFORE_REFETCH) {
+        conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
+        info(LD_REND, "Descriptor is here and fresh enough. Great.");
+        if (connection_ap_handshake_attach_circuit(conn) < 0) {
+          connection_mark_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
+          return -1;
+        }
+      } else {
+        conn->state = AP_CONN_STATE_RENDDESC_WAIT;
+        info(LD_REND, "Stale descriptor %s. Refetching.",
+             safe_str(conn->rend_query));
+        rend_client_refetch_renddesc(conn->rend_query);
+      }
+    }
+    return 0;
+  }
+  return 0; /* unreached but keeps the compiler happy */
+}
+
 /** connection_edge_process_inbuf() found a conn in state
  * socks_wait. See if conn->inbuf has the right bytes to proceed with
  * the socks handshake.
  *
- * If the handshake is complete, and it's for a general circuit, then
- * try to attach it to a circuit (or launch one as needed). If it's for
- * a rendezvous circuit, then fetch a rendezvous descriptor first (or
- * attach/launch a circuit if the rendezvous descriptor is already here
- * and fresh enough).
+ * If the handshake is complete, send it to
+ * connection_ap_handshake_rewrite_and_attach().
  *
  * Return -1 if an unexpected error with conn (and it should be marked
  * for close), else return 0.
@@ -942,9 +1132,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
 {
   socks_request_t *socks;
   int sockshere;
-  hostname_type_t addresstype;
   or_options_t *options = get_options();
-  int tor_should_handle_stream = !options->LeaveStreamsUnattached;
 
   tor_assert(conn);
   tor_assert(conn->type == CONN_TYPE_AP);
@@ -978,191 +1166,16 @@ connection_ap_handshake_process_socks(connection_t *conn)
     return -1;
   } /* else socks handshake is done, continue processing */
 
-  tor_strlower(socks->address); /* normalize it */
-  debug(LD_APP,"Client asked for %s:%d", safe_str(socks->address),
-        socks->port);
-
-  /* For address map controls, remap the address */
-  addressmap_rewrite(socks->address, sizeof(socks->address));
-
-  if (tor_should_handle_stream &&
-      address_is_in_virtual_range(socks->address)) {
-    /* This address was probably handed out by client_dns_get_unmapped_address,
-     * but the mapping was discarded for some reason.  We *don't* want to send
-     * the address through tor; that's likely to fail, and may leak
-     * information.
-     */
-    warn(LD_APP,"Missing mapping for virtual address '%s'. Refusing.",
-         socks->address); /* don't safe_str() this yet. */
-    connection_mark_unattached_ap(conn, END_STREAM_REASON_INTERNAL);
-    return -1;
-  }
-
-  /* Parse the address provided by SOCKS.  Modify it in-place if it
-   * specifies a hidden-service (.onion) or particular exit node (.exit).
-   */
-  addresstype = parse_extended_hostname(socks->address);
-
-  if (tor_should_handle_stream && addresstype == BAD_HOSTNAME) {
-    warn(LD_APP, "Invalid hostname %s; rejecting", socks->address);
-    connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
-    return -1;
-  }
-
-  if (addresstype == EXIT_HOSTNAME) {
-    /* foo.exit -- modify conn->chosen_exit_node to specify the exit
-     * node, and conn->address to hold only the address portion.*/
-    char *s = strrchr(socks->address,'.');
-    if (s) {
-      if (s[1] != '\0') {
-        conn->chosen_exit_name = tor_strdup(s+1);
-        *s = 0;
-      } else {
-        warn(LD_APP,"Malformed exit address '%s.exit'. Refusing.",
-             safe_str(socks->address));
-        connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
-        return -1;
-      }
-    } else {
-      struct in_addr in;
-      routerinfo_t *r;
-      conn->chosen_exit_name = tor_strdup(socks->address);
-      r = router_get_by_nickname(conn->chosen_exit_name, 1);
-      *socks->address = 0;
-      if (r) {
-        /* XXXX Should this use server->address instead? */
-        in.s_addr = htonl(r->addr);
-        strlcpy(socks->address, inet_ntoa(in), sizeof(socks->address));
-      } else if (tor_should_handle_stream) {
-        warn(LD_APP,
-             "Unrecognized server in exit address '%s.exit'. Refusing.",
-             safe_str(socks->address));
-        connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
-        return -1;
-      }
-    }
-  }
-
-  if (addresstype != ONION_HOSTNAME) {
-    /* not a hidden-service request (i.e. normal or .exit) */
-
-    if (tor_should_handle_stream &&
-        address_is_invalid_destination(socks->address)) {
-      warn(LD_APP,"Destination '%s' seems to be an invalid hostname. Failing.",
-           safe_str(socks->address));
-      connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
-      return -1;
-    }
-
-    if (socks->command == SOCKS_COMMAND_RESOLVE) {
-      uint32_t answer;
-      struct in_addr in;
-      /* Reply to resolves immediately if we can. */
-      if (strlen(socks->address) > RELAY_PAYLOAD_SIZE) {
-        warn(LD_APP,"Address to be resolved is too large. Failing.");
-        connection_ap_handshake_socks_resolved(conn,RESOLVED_TYPE_ERROR,
-                                               0,NULL,-1);
-        connection_mark_unattached_ap(conn,
-                                      END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
-        return -1;
-      }
-      if (tor_inet_aton(socks->address, &in)) { /* see if it's an IP already */
-        answer = in.s_addr; /* leave it in network order */
-        connection_ap_handshake_socks_resolved(conn,RESOLVED_TYPE_IPV4,4,
-                                               (char*)&answer,-1);
-        connection_mark_unattached_ap(conn,
-                                      END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
-        return 0;
-      }
-      rep_hist_note_used_resolve(time(NULL)); /* help predict this next time */
-      control_event_stream_status(conn, STREAM_EVENT_NEW_RESOLVE);
-    } else { /* socks->command == SOCKS_COMMAND_CONNECT */
-      if (socks->port == 0) {
-        notice(LD_APP,"Application asked to connect to port 0. Refusing.");
-        connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
-        return -1;
-      }
-
-      if (!conn->chosen_exit_name) {
-        /* see if we can find a suitable enclave exit */
-        routerinfo_t *r =
-          router_find_exact_exit_enclave(socks->address, socks->port);
-        if (r) {
-          info(LD_APP,"Redirecting address %s to exit at enclave router %s",
-               safe_str(socks->address), r->nickname);
-          /* use the hex digest, not nickname, in case there are two
-             routers with this nickname */
-          conn->chosen_exit_name =
-            tor_strdup(hex_str(r->cache_info.identity_digest, DIGEST_LEN));
-        }
-      }
-
-      /* help predict this next time */
-      rep_hist_note_used_port(socks->port, time(NULL));
-      control_event_stream_status(conn, STREAM_EVENT_NEW);
-    }
-    if (!tor_should_handle_stream) {
-      conn->state = AP_CONN_STATE_CONTROLLER_WAIT;
-    } else {
-      conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
-      if (connection_ap_handshake_attach_circuit(conn) < 0) {
-        connection_mark_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
-        return -1;
-      }
-    }
-    return 0;
-  } else {
-    /* it's a hidden-service request */
-    rend_cache_entry_t *entry;
-    int r;
-
-    if (socks->command == SOCKS_COMMAND_RESOLVE) {
-      /* if it's a resolve request, fail it right now, rather than
-       * building all the circuits and then realizing it won't work. */
-      warn(LD_APP,"Resolve requests to hidden services not allowed. Failing.");
-      connection_ap_handshake_socks_resolved(conn,RESOLVED_TYPE_ERROR,
-                                             0,NULL,-1);
-      connection_mark_unattached_ap(conn,
-                                    END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
-      return -1;
-    }
-
-    strlcpy(conn->rend_query, socks->address, sizeof(conn->rend_query));
-    info(LD_REND,"Got a hidden service request for ID '%s'",
-         safe_str(conn->rend_query));
-    /* see if we already have it cached */
-    r = rend_cache_lookup_entry(conn->rend_query, -1, &entry);
-    if (r<0) {
-      warn(LD_REND,"Invalid service descriptor %s",
-           safe_str(conn->rend_query));
-      connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
-      return -1;
-    }
-    if (r==0) {
-      conn->state = AP_CONN_STATE_RENDDESC_WAIT;
-      info(LD_REND, "Unknown descriptor %s. Fetching.",
-           safe_str(conn->rend_query));
-      rend_client_refetch_renddesc(conn->rend_query);
-    } else { /* r > 0 */
-#define NUM_SECONDS_BEFORE_REFETCH (60*15)
-      if (time(NULL) - entry->received < NUM_SECONDS_BEFORE_REFETCH) {
-        conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
-        info(LD_REND, "Descriptor is here and fresh enough. Great.");
-        if (connection_ap_handshake_attach_circuit(conn) < 0) {
-          connection_mark_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
-          return -1;
-        }
-      } else {
-        conn->state = AP_CONN_STATE_RENDDESC_WAIT;
-        info(LD_REND, "Stale descriptor %s. Refetching.",
-             safe_str(conn->rend_query));
-        rend_client_refetch_renddesc(conn->rend_query);
-      }
-    }
+  if (socks->command == SOCKS_COMMAND_CONNECT)
     control_event_stream_status(conn, STREAM_EVENT_NEW);
+  else
+    control_event_stream_status(conn, STREAM_EVENT_NEW_RESOLVE);
+
+  if (options->LeaveStreamsUnattached) {
+    conn->state = AP_CONN_STATE_CONTROLLER_WAIT;
     return 0;
-  }
-  return 0; /* unreached but keeps the compiler happy */
+  } else
+    return connection_ap_handshake_rewrite_and_attach(conn);
 }
 
 /** Iterate over the two bytes of stream_id until we get one that is not
