@@ -1,8 +1,8 @@
-/* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: control.c,v 1.164 2006/01/10 22:42:44 arma Exp $ */
+/* $Id: control.c,v 1.168 2006/02/09 05:46:48 arma Exp $ */
 const char control_c_id[] =
-  "$Id: control.c,v 1.164 2006/01/10 22:42:44 arma Exp $";
+  "$Id: control.c,v 1.168 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file control.c
@@ -1116,7 +1116,7 @@ handle_control_signal(connection_t *conn, uint32_t len,
       sig = SIGTERM;
     else {
       connection_printf_to_buf(conn, "552 Unrecognized signal code \"%s\"\r\n",
-                               body);
+                               s);
       sig = -1;
     }
     tor_free(s);
@@ -1279,20 +1279,27 @@ handle_getinfo_helper(const char *question, char **answer)
     *answer = list_getinfo_options();
   } else if (!strcmpstart(question, "desc/id/")) {
     routerinfo_t *ri = router_get_by_hexdigest(question+strlen("desc/id/"));
-    if (ri && ri->cache_info.signed_descriptor)
-      *answer = tor_strdup(ri->cache_info.signed_descriptor);
+    if (ri) {
+      const char *body = signed_descriptor_get_body(&ri->cache_info);
+      if (body)
+        *answer = tor_strdup(body);
+    }
   } else if (!strcmpstart(question, "desc/name/")) {
     routerinfo_t *ri = router_get_by_nickname(question+strlen("desc/name/"),1);
-    if (ri && ri->cache_info.signed_descriptor)
-      *answer = tor_strdup(ri->cache_info.signed_descriptor);
+    if (ri) {
+      const char *body = signed_descriptor_get_body(&ri->cache_info);
+      if (body)
+        *answer = tor_strdup(body);
+    }
   } else if (!strcmp(question, "desc/all-recent")) {
     routerlist_t *routerlist = router_get_routerlist();
     smartlist_t *sl = smartlist_create();
     if (routerlist && routerlist->routers) {
       SMARTLIST_FOREACH(routerlist->routers, routerinfo_t *, ri,
       {
-      if (ri && ri->cache_info.signed_descriptor)
-        smartlist_add(sl, tor_strdup(ri->cache_info.signed_descriptor));
+        const char *body = signed_descriptor_get_body(&ri->cache_info);
+        if (body)
+          smartlist_add(sl, tor_strdup(body));
       });
     }
     *answer = smartlist_join_strings(sl, "", 0, NULL);
@@ -1737,9 +1744,7 @@ handle_control_attachstream(connection_t *conn, uint32_t len,
   }
 
   if (zero_circ) {
-    ap_conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
-    if (connection_ap_handshake_attach_circuit(ap_conn)<0)
-      connection_mark_unattached_ap(ap_conn, END_STREAM_REASON_CANT_ATTACH);
+    connection_ap_handshake_rewrite_and_attach(ap_conn);
     send_control_done(conn);
     return 0;
   }
