@@ -1,9 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.341 2006/01/10 20:50:22 nickm Exp $ */
+/* $Id: directory.c,v 1.350 2006/02/09 05:46:48 arma Exp $ */
 const char directory_c_id[] =
-  "$Id: directory.c,v 1.341 2006/01/10 20:50:22 nickm Exp $";
+  "$Id: directory.c,v 1.350 2006/02/09 05:46:48 arma Exp $";
 
 #include "or.h"
 
@@ -131,15 +131,20 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
 {
   smartlist_t *dirservers;
   int post_via_tor;
+  int post_to_v1_only;
 
   router_get_trusted_dir_servers(&dirservers);
   tor_assert(dirservers);
+  /* Only old dirservers handle rendezvous descriptor publishing. */
+  post_to_v1_only = (purpose == DIR_PURPOSE_UPLOAD_RENDDESC);
   /* This tries dirservers which we believe to be down, but ultimately, that's
    * harmless, and we may as well err on the side of getting things uploaded.
    */
   SMARTLIST_FOREACH(dirservers, trusted_dir_server_t *, ds,
     {
       routerstatus_t *rs = &(ds->fake_status);
+      if (post_to_v1_only && !ds->is_v1_authority)
+        continue;
       post_via_tor = purpose_is_private(purpose) ||
                      !fascist_firewall_allows_address(ds->addr,ds->dir_port);
       directory_initiate_command_routerstatus(rs, purpose, post_via_tor,
@@ -332,7 +337,7 @@ connection_dir_download_networkstatus_failed(connection_t *conn)
     directory_get_from_dirserver(conn->purpose, "all.z",
                                  0 /* don't retry_if_no_servers */);
   } else if (!strcmpstart(conn->requested_resource, "fp/")) {
-    /* We were trying to download by fingerprint; mark them all has having
+    /* We were trying to download by fingerprint; mark them all as having
      * failed, and possibly retry them later.*/
     smartlist_t *failed = smartlist_create();
     dir_split_resource_into_fingerprints(conn->requested_resource+3,
@@ -579,6 +584,14 @@ directory_send_command(connection_t *conn, const char *platform,
       tor_assert(0);
       return;
   }
+
+  if (strlen(proxystring) + strlen(url) >= 4096) {
+    warn(LD_BUG,
+         "Bug: squid does not like URLs longer than 4095 bytes, this "
+         "one is %d bytes long: %s%s",
+         (int)(strlen(proxystring) + strlen(url)), proxystring, url);
+  }
+
   tor_snprintf(request, sizeof(request), "%s %s", httpcommand, proxystring);
   connection_write_to_buf(request, strlen(request), conn);
   connection_write_to_buf(url, strlen(url), conn);
@@ -891,6 +904,14 @@ connection_dir_client_reached_eof(connection_t *conn)
     }
   }
 
+  if (status_code == 503) {
+    info(LD_DIR,"Received http status code %d (\"%s\") from server "
+         "'%s:%d'. I'll try again soon.",
+         status_code, reason, conn->address, conn->port);
+    tor_free(body); tor_free(headers); tor_free(reason);
+    return -1;
+  }
+
   plausible = body_is_plausible(body, body_len, conn->purpose);
   if (compression || !plausible) {
     char *new_body = NULL;
@@ -950,12 +971,6 @@ connection_dir_client_reached_eof(connection_t *conn)
     /* fetch/process the directory to cache it. */
     info(LD_DIR,"Received directory (size %d) from server '%s:%d'",
          (int)body_len, conn->address, conn->port);
-    if (status_code == 503 || body_len == 0) {
-      info(LD_DIR,"Empty directory; status %d (\"%s\") Ignoring.",
-           status_code, reason);
-      tor_free(body); tor_free(headers); tor_free(reason);
-      return -1;
-    }
     if (status_code != 200) {
       warn(LD_DIR,"Received http status code %d (\"%s\") from server "
            "'%s:%d'. I'll try again soon.",
@@ -1231,7 +1246,8 @@ connection_dir_process_inbuf(connection_t *conn)
 
   /* XXX for READ states, might want to make sure inbuf isn't too big */
 
-  debug(LD_HTTP,"Got data, not eof. Leaving on inbuf.");
+  if (!conn->inbuf_reached_eof)
+    debug(LD_HTTP,"Got data, not eof. Leaving on inbuf.");
   return 0;
 }
 
@@ -1516,8 +1532,8 @@ directory_handle_command_get(connection_t *conn, char *headers,
         char *cp = inp;
         SMARTLIST_FOREACH(descs, signed_descriptor_t *, ri,
            {
-             memcpy(cp, ri->signed_descriptor,
-                    ri->signed_descriptor_len);
+             const char *body = signed_descriptor_get_body(ri);
+             memcpy(cp, body, ri->signed_descriptor_len);
              cp += ri->signed_descriptor_len;
              *cp++ = '\n';
            });
@@ -1550,9 +1566,10 @@ directory_handle_command_get(connection_t *conn, char *headers,
         note_request(request_type, len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
         SMARTLIST_FOREACH(descs, signed_descriptor_t *, ri,
-                          connection_write_to_buf(ri->signed_descriptor,
-                                                  ri->signed_descriptor_len,
-                                                  conn));
+          {
+            const char *body = signed_descriptor_get_body(ri);
+            connection_write_to_buf(body, ri->signed_descriptor_len, conn);
+          });
       }
     }
     smartlist_free(descs);
@@ -1795,7 +1812,7 @@ connection_dir_finished_connecting(connection_t *conn)
 }
 
 /** Called when one or more networkstatus fetches have failed (with uppercase
- * fingerprints listed in <b>failed</>).  Mark those fingerprints has having
+ * fingerprints listed in <b>failed</>).  Mark those fingerprints as having
  * failed once. */
 static void
 dir_networkstatus_download_failed(smartlist_t *failed)
