@@ -1,9 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.291 2006/01/11 03:59:55 nickm Exp $ */
+/* $Id: dirserv.c,v 1.297 2006/02/09 05:46:49 arma Exp $ */
 const char dirserv_c_id[] =
-  "$Id: dirserv.c,v 1.291 2006/01/11 03:59:55 nickm Exp $";
+  "$Id: dirserv.c,v 1.297 2006/02/09 05:46:49 arma Exp $";
 
 #include "or.h"
 
@@ -737,6 +737,7 @@ list_server_status(smartlist_t *routers, char **router_status_out)
    * equals-suffixed nickname, then a dollar-prefixed hexdigest. */
   smartlist_t *rs_entries;
   time_t now = time(NULL);
+  time_t cutoff = now - ROUTER_MAX_AGE;
   int authdir_mode = get_options()->AuthoritativeDir;
   tor_assert(router_status_out);
 
@@ -748,7 +749,8 @@ list_server_status(smartlist_t *routers, char **router_status_out)
       /* Update router status in routerinfo_t. */
       ri->is_running = dirserv_thinks_router_is_reachable(ri, now);
     }
-    smartlist_add(rs_entries, list_single_server_status(ri, ri->is_running));
+    if (ri->cache_info.published_on >= cutoff)
+      smartlist_add(rs_entries, list_single_server_status(ri, ri->is_running));
   });
 
   *router_status_out = smartlist_join_strings(rs_entries, " ", 0, NULL);
@@ -759,30 +761,6 @@ list_server_status(smartlist_t *routers, char **router_status_out)
   return 0;
 }
 
-/** Helper: Given pointers to two strings describing tor versions, return -1
- * if _a precedes _b, 1 if _b preceeds _a, and 0 if they are equivalent.
- * Used to sort a list of versions. */
-static int
-_compare_tor_version_str_ptr(const void **_a, const void **_b)
-{
-  const char *a = *_a, *b = *_b;
-  int ca, cb;
-  tor_version_t va, vb;
-  ca = tor_version_parse(a, &va);
-  cb = tor_version_parse(b, &vb);
-  /* If they both parse, compare them. */
-  if (!ca && !cb)
-    return tor_version_compare(&va,&vb);
-  /* If one parses, it comes first. */
-  if (!ca && cb)
-    return -1;
-  if (ca && !cb)
-    return 1;
-  /* If neither parses, compare strings.  Also, the directory server admin
-  ** needs to be smacked upside the head.  But Tor is tolerant and gentle. */
-  return strcmp(a,b);
-}
-
 /* Given a (possibly empty) list of config_line_t, each line of which contains
  * a list of comma-separated version numbers surrounded by optional space,
  * allocate and return a new string containing the version numbers, in order,
@@ -798,7 +776,7 @@ format_versions_list(config_line_t *ln)
     smartlist_split_string(versions, ln->value, ",",
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
   }
-  smartlist_sort(versions, _compare_tor_version_str_ptr);
+  sort_version_list(versions);
   result = smartlist_join_strings(versions,",",0,NULL);
   SMARTLIST_FOREACH(versions,char *,s,tor_free(s));
   smartlist_free(versions);
@@ -871,9 +849,11 @@ dirserv_dump_directory_to_string(char **dir_out,
   SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri,
     {
       size_t len = ri->cache_info.signed_descriptor_len;
+      const char *body;
       if (cp+len+1 >= buf+buf_len)
         goto truncated;
-      memcpy(cp, ri->cache_info.signed_descriptor, len);
+      body = signed_descriptor_get_body(&ri->cache_info);
+      memcpy(cp, body, len);
       cp += len;
       *cp++ = '\n'; /* add an extra newline in case somebody was depending on
                      * it. */
@@ -881,8 +861,7 @@ dirserv_dump_directory_to_string(char **dir_out,
   *cp = '\0';
 
   /* These multiple strlcat calls are inefficient, but dwarfed by the RSA
-     signature.
-  */
+     signature. */
   if (strlcat(buf, "directory-signature ", buf_len) >= buf_len)
     goto truncated;
   if (strlcat(buf, get_options()->Nickname, buf_len) >= buf_len)
@@ -1343,6 +1322,7 @@ generate_v2_networkstatus(void)
   crypto_pk_env_t *private_key = get_identity_key();
   routerlist_t *rl = router_get_routerlist();
   time_t now = time(NULL);
+  time_t cutoff = now - ROUTER_MAX_AGE;
   int naming = options->NamingAuthoritativeDir;
   int versioning = options->VersioningAuthoritativeDir;
   const char *contact;
@@ -1410,6 +1390,7 @@ generate_v2_networkstatus(void)
   dirserv_compute_stable_uptime(rl);
 
   SMARTLIST_FOREACH(rl->routers, routerinfo_t *, ri, {
+    if (ri->cache_info.published_on >= cutoff) {
       int f_exit = router_is_general_exit(ri);
       int f_stable = ri->is_stable =
                      !dirserv_thinks_router_is_unreliable(ri, 1, 0);
@@ -1420,6 +1401,7 @@ generate_v2_networkstatus(void)
                                       ri->cache_info.identity_digest);
       int f_named = naming && ri->is_named;
       int f_valid = ri->is_verified;
+      int f_guard = f_fast && f_stable;
       /* 0.1.1.9-alpha is the first version to support fetch by descriptor
        * hash. */
       int f_v2_dir = ri->dir_port &&
@@ -1437,7 +1419,7 @@ generate_v2_networkstatus(void)
 
       if (tor_snprintf(outp, endp-outp,
                        "r %s %s %s %s %s %d %d\n"
-                       "s%s%s%s%s%s%s%s%s\n",
+                       "s%s%s%s%s%s%s%s%s%s\n",
                        ri->nickname,
                        identity64,
                        digest64,
@@ -1448,6 +1430,7 @@ generate_v2_networkstatus(void)
                        f_authority?" Authority":"",
                        f_exit?" Exit":"",
                        f_fast?" Fast":"",
+                       f_guard?" Guard":"",
                        f_named?" Named":"",
                        f_stable?" Stable":"",
                        f_running?" Running":"",
@@ -1457,7 +1440,8 @@ generate_v2_networkstatus(void)
         goto done;
       }
       outp += strlen(outp);
-    });
+    }
+  });
 
   if (tor_snprintf(outp, endp-outp, "directory-signature %s\n",
                    get_options()->Nickname)<0) {
