@@ -1,10 +1,10 @@
-/* Copyright 2001 Matej Pfajfar.
- * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.614 2006/01/11 22:34:07 arma Exp $ */
+/* $Id: main.c,v 1.622 2006/02/09 05:46:49 arma Exp $ */
 const char main_c_id[] =
-  "$Id: main.c,v 1.614 2006/01/11 22:34:07 arma Exp $";
+  "$Id: main.c,v 1.622 2006/02/09 05:46:49 arma Exp $";
 
 /**
  * \file main.c
@@ -482,6 +482,7 @@ conn_close_if_marked(int i)
     if (connection_wants_to_flush(conn)) {
       int severity;
       if (conn->type == CONN_TYPE_EXIT ||
+          (conn->type == CONN_TYPE_OR && server_mode(get_options())) ||
           (conn->type == CONN_TYPE_DIR && conn->purpose == DIR_PURPOSE_SERVER))
         severity = LOG_INFO;
       else
@@ -536,7 +537,7 @@ get_dir_fetch_period(or_options_t *options)
     return 20*60;
   else
     /* Default for average user. */
-    return 40*60;
+    return 120*60;
 }
 
 /**
@@ -567,6 +568,7 @@ directory_info_has_arrived(time_t now, int from_cache)
     log(LOG_NOTICE, LD_DIR,
         "I learned some more directory information, but not enough to "
         "build a circuit.");
+    update_router_descriptor_downloads(now);
     return;
   }
 
@@ -574,7 +576,7 @@ directory_info_has_arrived(time_t now, int from_cache)
       !we_are_hibernating()) { /* connect to the appropriate routers */
     if (!authdir_mode(options))
       router_retry_connections(0, 1);
-    if (!from_cache)
+    if (!from_cache && has_completed_circuit)
       consider_testing_reachability();
   }
 }
@@ -786,6 +788,8 @@ run_scheduled_events(time_t now)
 
     /* Only caches actually need to fetch directories now. */
     if (options->DirPort && !options->V1AuthoritativeDir) {
+      /* XXX actually, we should only do this if we want to advertise
+       * our dirport. not simply if we configured one. -RD */
       directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
     }
 
@@ -823,6 +827,7 @@ run_scheduled_events(time_t now)
     /* also, check religiously for reachability, if it's within the first
      * 20 minutes of our uptime. */
     if (server_mode(options) &&
+        has_completed_circuit &&
         stats_n_seconds_working < TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT &&
         !we_are_hibernating())
       consider_testing_reachability();
@@ -891,7 +896,7 @@ run_scheduled_events(time_t now)
   circuit_close_all_marked();
 
   /** 7. And upload service descriptors if necessary. */
-  if (have_dir_info && !we_are_hibernating())
+  if (has_completed_circuit && !we_are_hibernating())
     rend_consider_services_upload(now);
 
   /** 8. and blow away any connections that need to die. have to do this now,
@@ -1011,7 +1016,7 @@ do_hup(void)
   char keydir[512];
   or_options_t *options = get_options();
 
-  notice(LD_GENERAL,"Received sighup. Reloading config.");
+  notice(LD_GENERAL,"Received reload signal (hup). Reloading config.");
   has_completed_circuit=0;
   if (accounting_is_enabled(options))
     accounting_record_bandwidth_usage(time(NULL));
@@ -1138,10 +1143,10 @@ do_main_loop(void)
       /* let the program survive things like ^z */
       if (e != EINTR && !ERRNO_IS_EINPROGRESS(e)) {
 #ifdef HAVE_EVENT_GET_METHOD
-        err(LD_NET,"libevent poll with %s failed: %s [%d]",
+        err(LD_NET,"libevent call with %s failed: %s [%d]",
             event_get_method(), tor_socket_strerror(e), e);
 #else
-        err(LD_NET,"libevent poll failed: %s [%d]",
+        err(LD_NET,"libevent call failed: %s [%d]",
             tor_socket_strerror(e), e);
 #endif
         return -1;
@@ -1153,8 +1158,8 @@ do_main_loop(void)
 #endif
       } else {
         if (ERRNO_IS_EINPROGRESS(e))
-          warn(LD_BUG,"libevent poll returned EINPROGRESS? Please report.");
-        debug(LD_NET,"event poll interrupted.");
+          warn(LD_BUG,"libevent call returned EINPROGRESS? Please report.");
+        debug(LD_NET,"libevent call interrupted.");
         /* You can't trust the results of this poll(). Go back to the
          * top of the big for loop. */
         continue;
