@@ -1,10 +1,10 @@
-/* Copyright 2001 Matej Pfajfar.
- * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.239 2006/01/10 07:21:01 arma Exp $ */
+/* $Id: router.c,v 1.243 2006/02/09 05:46:49 arma Exp $ */
 const char router_c_id[] =
-  "$Id: router.c,v 1.239 2006/01/10 07:21:01 arma Exp $";
+  "$Id: router.c,v 1.243 2006/02/09 05:46:49 arma Exp $";
 
 #include "or.h"
 
@@ -63,6 +63,7 @@ dup_onion_keys(crypto_pk_env_t **key, crypto_pk_env_t **last)
   tor_assert(key);
   tor_assert(last);
   tor_mutex_acquire(key_lock);
+  tor_assert(onionkey);
   *key = crypto_pk_dup_key(onionkey);
   if (lastonionkey)
     *last = crypto_pk_dup_key(lastonionkey);
@@ -761,13 +762,14 @@ router_get_my_routerinfo(void)
 const char *
 router_get_my_descriptor(void)
 {
+  const char *body;
   if (!desc_routerinfo) {
     if (router_rebuild_descriptor(1))
       return NULL;
   }
-  debug(LD_GENERAL,"my desc is '%s'",
-        desc_routerinfo->cache_info.signed_descriptor);
-  return desc_routerinfo->cache_info.signed_descriptor;
+  body = signed_descriptor_get_body(&desc_routerinfo->cache_info);
+  debug(LD_GENERAL,"my desc is '%s'", body);
+  return body;
 }
 
 /*DOCDOC*/
@@ -818,8 +820,8 @@ router_rebuild_descriptor(int force)
   if (options->BandwidthRate > options->MaxAdvertisedBandwidth)
     ri->bandwidthrate = (int)options->MaxAdvertisedBandwidth;
 
-  config_parse_addr_policy(get_options()->ExitPolicy, &ri->exit_policy, -1);
-  options_append_default_exit_policy(&ri->exit_policy);
+  config_parse_exit_policy(options->ExitPolicy, &ri->exit_policy,
+                           options->ExitPolicyRejectPrivate);
 
   if (desc_routerinfo) { /* inherit values */
     ri->is_verified = desc_routerinfo->is_verified;
@@ -865,16 +867,16 @@ router_rebuild_descriptor(int force)
      });
     smartlist_free(family);
   }
-  ri->cache_info.signed_descriptor = tor_malloc(8192);
-  if (router_dump_router_to_string(ri->cache_info.signed_descriptor, 8192,
+  ri->cache_info.signed_descriptor_body = tor_malloc(8192);
+  if (router_dump_router_to_string(ri->cache_info.signed_descriptor_body, 8192,
                                    ri, get_identity_key())<0) {
     warn(LD_BUG, "Couldn't allocate string for descriptor.");
     return -1;
   }
   ri->cache_info.signed_descriptor_len =
-    strlen(ri->cache_info.signed_descriptor);
+    strlen(ri->cache_info.signed_descriptor_body);
   crypto_digest(ri->cache_info.signed_descriptor_digest,
-                ri->cache_info.signed_descriptor,
+                ri->cache_info.signed_descriptor_body,
                 ri->cache_info.signed_descriptor_len);
 
   if (desc_routerinfo)
