@@ -1,10 +1,10 @@
-/* Copyright 2001 Matej Pfajfar.
- * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.167 2006/01/02 05:11:31 arma Exp $ */
+/* $Id: routerparse.c,v 1.173 2006/02/09 05:46:49 arma Exp $ */
 const char routerparse_c_id[] =
-  "$Id: routerparse.c,v 1.167 2006/01/02 05:11:31 arma Exp $";
+  "$Id: routerparse.c,v 1.173 2006/02/09 05:46:49 arma Exp $";
 
 /**
  * \file routerparse.c
@@ -367,7 +367,7 @@ router_parse_directory(const char *str)
 
   /* XXXX This could be simplified a lot, but it will all go away
    * once pre-0.1.1.8 is obsolete, and for now it's better not to
-   * tuoch it. */
+   * touch it. */
 
   if (router_get_dir_hash(str, digest)) {
     warn(LD_DIR, "Unable to compute digest of directory");
@@ -745,7 +745,7 @@ router_parse_entry_from_string(const char *s, const char *end)
   }
 
   router = tor_malloc_zero(sizeof(routerinfo_t));
-  router->cache_info.signed_descriptor = tor_strndup(s, end-s);
+  router->cache_info.signed_descriptor_body = tor_strndup(s, end-s);
   router->cache_info.signed_descriptor_len = end-s;
   memcpy(router->cache_info.signed_descriptor_digest, digest, DIGEST_LEN);
 
@@ -1037,6 +1037,8 @@ routerstatus_parse_entry_from_string(const char **s, smartlist_t *tokens)
         rs->is_valid = 1;
       else if (!strcmp(tok->args[i], "V2Dir"))
         rs->is_v2_dir = 1;
+      else if (!strcmp(tok->args[i], "Guard"))
+        rs->is_possible_guard = 1;
     }
   }
 
@@ -1649,7 +1651,7 @@ find_all_exitpolicy(smartlist_t *s)
   return out;
 }
 
-/** Compute the SHA digest of the substring of <b>s</b> taken from the first
+/** Compute the SHA-1 digest of the substring of <b>s</b> taken from the first
  * occurrence of <b>start_str</b> through the first newline after the first
  * subsequent occurrence of <b>end_str</b>; store the 20-byte result in
  * <b>digest</b>; return 0 on success.
@@ -1833,3 +1835,34 @@ tor_version_same_series(tor_version_t *a, tor_version_t *b)
           (a->micro == b->micro));
 }
 
+/** Helper: Given pointers to two strings describing tor versions, return -1
+ * if _a precedes _b, 1 if _b preceeds _a, and 0 if they are equivalent.
+ * Used to sort a list of versions. */
+static int
+_compare_tor_version_str_ptr(const void **_a, const void **_b)
+{
+  const char *a = *_a, *b = *_b;
+  int ca, cb;
+  tor_version_t va, vb;
+  ca = tor_version_parse(a, &va);
+  cb = tor_version_parse(b, &vb);
+  /* If they both parse, compare them. */
+  if (!ca && !cb)
+    return tor_version_compare(&va,&vb);
+  /* If one parses, it comes first. */
+  if (!ca && cb)
+    return -1;
+  if (ca && !cb)
+    return 1;
+  /* If neither parses, compare strings.  Also, the directory server admin
+  ** needs to be smacked upside the head.  But Tor is tolerant and gentle. */
+  return strcmp(a,b);
+}
+
+/** Sort a list of string-representations of versions in ascending order. */
+void
+sort_version_list(smartlist_t *versions)
+{
+  smartlist_sort(versions, _compare_tor_version_str_ptr);
+}
+
