@@ -1,9 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: test.c,v 1.217 2006/01/10 20:47:24 nickm Exp $ */
+/* $Id: test.c,v 1.219 2006/02/09 05:46:49 arma Exp $ */
 const char test_c_id[] =
-  "$Id: test.c,v 1.217 2006/01/10 20:47:24 nickm Exp $";
+  "$Id: test.c,v 1.219 2006/02/09 05:46:49 arma Exp $";
 
 /**
  * \file test.c
@@ -1438,7 +1438,7 @@ test_exit_policies(void)
   test_eq(65535, policy->prt_max);
   test_streq("reject 192.168.0.0/16:*", policy->string);
 
-  test_assert(exit_policy_implicitly_allows_local_networks(policy, 0));
+//  test_assert(exit_policy_implicitly_allows_local_networks(policy, 0));
   test_assert(ADDR_POLICY_ACCEPTED ==
           router_compare_addr_to_addr_policy(0x01020304u, 2, policy));
   test_assert(ADDR_POLICY_PROBABLY_ACCEPTED ==
@@ -1448,6 +1448,7 @@ test_exit_policies(void)
 
   addr_policy_free(policy);
 
+#if 0
   /* Copied from router.c */
   policy = NULL;
   options_append_default_exit_policy(&policy);
@@ -1455,6 +1456,7 @@ test_exit_policies(void)
   test_assert(!exit_policy_implicitly_allows_local_networks(policy, 1));
 
   addr_policy_free(policy);
+#endif
 
 }
 
